
//+-----------------------------------------------------------------+
//|                                             macd cross zero.mq4 |
//|                                             by whomever         |
//|                                                                 |
//+-----------------------------------------------------------------+

#property indicator_separate_window
#property indicator_buffers    4
#property indicator_color1     Aqua
#property indicator_color2     Red
#property indicator_color3     Green
#property indicator_color4     Red
#property indicator_width1     2
#property indicator_levelcolor DarkSlateGray

//
//
//
//
//

extern string TimeFrame          = "Current time frame";
extern int    FastEMA            = 12;
extern int    SlowEMA            = 26;
extern int    SignalSMA          = 9;
extern bool   ShowArrowsOnly     = true;

extern string note               = "turn on Alert = true; turn off = false";
extern bool   alertsOn           = true;
extern bool   alertsOnCurrent    = true;
extern bool   alertsMessage      = true;
extern bool   alertsSound        = true;
extern bool   alertsEmail        = false;
extern string soundfile          = "alert2.wav";

//
//
//
//
//

double macd[];
double signal[];
double UpArrow[];     
double DnArrow[];
double trend[];   

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);  
   SetIndexBuffer(0,macd);    SetIndexStyle(0,DRAW_HISTOGRAM); 
   SetIndexBuffer(1,signal);                                   
   SetIndexBuffer(2,UpArrow); SetIndexStyle(2,DRAW_ARROW); SetIndexArrow(2,233);
   SetIndexBuffer(3,DnArrow); SetIndexStyle(3,DRAW_ARROW); SetIndexArrow(3,234);
   SetIndexBuffer(4,trend);
   SetLevelValue(0,0);
   
   if (ShowArrowsOnly)
   {
     SetIndexStyle(0,DRAW_NONE);
     SetIndexStyle(1,DRAW_NONE);
   }
   else
   {
     SetIndexStyle(0,DRAW_HISTOGRAM); SetIndexLabel(0,"MACD");
     SetIndexStyle(1,DRAW_LINE);      SetIndexLabel(1,"Signal");
   }       
    
   //
   //
   //
   //
   //
   
      FastEMA           = MathMax(FastEMA,1);
      SlowEMA           = MathMax(SlowEMA,1);
      SignalSMA         = MathMax(SignalSMA,1);
      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame == "returnBars");     if (returnBars)     return(0);
      calculateValue    = (TimeFrame == "calculateValue"); if (calculateValue) return(0);
      timeFrame         = stringToTimeFrame(TimeFrame);   

   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(timeFrame)+"  MACD("+FastEMA+", "+SlowEMA+", "+SignalSMA+")");
   return(0);
}
      
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int i,limit;
   int counted_bars = IndicatorCounted();

   if(counted_bars < 0) return(-1);
   if(counted_bars > 0) counted_bars--;
             limit=MathMin(Bars-counted_bars,Bars-1);
             if (returnBars) { macd[0] = limit; return(0); }

   //
   //
   //
   //
   //

   if (calculateValue || timeFrame == Period())
   {
      
         for (i=limit; i>=0; i--) macd[i]   = iMA(NULL,0,FastEMA,0,MODE_EMA,PRICE_CLOSE,i)- iMA(NULL,0,SlowEMA,0,MODE_EMA,PRICE_CLOSE,i);
         for (i=limit; i>=0; i--) signal[i] = iMAOnArray(macd,Bars,SignalSMA,0,MODE_EMA,i);
         for (i=limit; i>=0; i--) 
         {
          
      
         trend[i] = trend[i+1];
            if (macd[i] > 0) trend[i] = 1;
            if (macd[i] < 0) trend[i] =-1;
         
            //
            //
            //
            //
            //
      
            UpArrow[i] = EMPTY_VALUE;
            DnArrow[i] = EMPTY_VALUE;
            if (trend[i]!= trend[i+1])
            if (trend[i] == 1)
                  UpArrow[i] = -0.0002;
            else  DnArrow[i] =  0.0002; 
      }
      manageAlerts();
      return(0);
      }
   
      //
      //
      //
      //
      //
  
      limit = MathMax(limit,MathMin(Bars,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
      for (i=limit;i>=0; i--)
      {
         int y = iBarShift(NULL,timeFrame,Time[i]);
            macd[i]    = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalSMA,0,y);
            signal[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalSMA,1,y);
            UpArrow[i] = EMPTY_VALUE;
            DnArrow[i] = EMPTY_VALUE;
            trend[i]   = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",FastEMA,SlowEMA,SignalSMA,4,y);
            if (trend[i]!= trend[i+1])
            if (trend[i] == 1)
                  UpArrow[i] = -0.0002;
            else  DnArrow[i] =  0.0002;
            
            
      }
  manageAlerts();
return(0);
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

//
//
//
//
//

void manageAlerts()
{
   if (!calculateValue && alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] == 1) doAlert(whichBar,"buy");
         if (trend[whichBar] ==-1) doAlert(whichBar,"sell");
      }         
   }
}   

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," - ",timeFrameToString(timeFrame)+" Macd Cross zero separate ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol()," Macd Cross zero separate "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}